//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Blikanie diody riadene casovacom RTT
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

#define RTT_INTERRUPT_LEVEL 0       // 0 - najnizsia priorita prerusenia

//  *******************************************************
//  Obsluha prerusenia od casovaca realneho casu RTT
//  *******************************************************
void RTT_prerusenie(void)
{
  // vymazanie ziadosti o prerusenie
  AT91C_BASE_RTTC->RTTC_RTSR;

  // prepnutie LED diody
  if (AT91C_BASE_PIOA->PIO_PDSR & D1)
    AT91C_BASE_PIOA->PIO_CODR = D1;	      // vypnutie LED
  else
    AT91C_BASE_PIOA->PIO_SODR = D1;	      // zapnutie LED
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{
  //******************* Uvodna inicializacia *******************

  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************

  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = D1;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = D1;

  // vsetko je vypnute
  AT91C_BASE_PIOA->PIO_CODR = D1;

  //******************* Nastavenie prerusenia od RTT ******************
  // Povolenie hodin pre RTT
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_SYS; 
  // nastavenie delicky na 0.5s (max je 2s) - do RTT vstupuje SCK (32768 Hz)
  // povolenie generovania preruseni, reset citaca
  AT91C_BASE_RTTC->RTTC_RTMR = 16384 | AT91C_RTTC_RTTRST | AT91C_RTTC_RTTINCIEN;

  // nastavenie obsluzneho programu
  AT91C_BASE_AIC->AIC_SVR[AT91C_ID_SYS] = (unsigned int) RTT_prerusenie;
  // zdroj prerusenia a priorita
  AT91C_BASE_AIC->AIC_SMR[AT91C_ID_SYS] = RTT_INTERRUPT_LEVEL;
  // zrusenie prerusenia (ak nejake nahodou bolo)
  AT91C_BASE_AIC->AIC_ICCR = 0x1 << AT91C_ID_SYS;
  // povolenie prerusenia od RTT
  AT91C_BASE_AIC->AIC_IECR = 0x1 << AT91C_ID_SYS;

  //******************* Hlavna cast programu *******************

  // zapnutie LED - bola ukoncena inicializacia
  AT91C_BASE_PIOA->PIO_SODR = D1;

  // nekonecna slucka
  while(1);
}
